#include "Main.h"
#include "Map.h"
#include "MapData.h"
#include "Block.h"



// ----- Local function prototypes -----

static void BlockSetMode (Sprite *spt, u32 mode, u32 state);



// ----- Global functions -----

void SptInitBlock (Sprite *spt, Map *map)
{
	BlockSetMode (spt, BLOCK_MODE_IDLE, BLOCK_IDLE_STATE_FULL);
}

void SptUpdateBlock (Sprite *spt, Map *map)
{
	switch (spt->vars.block.mode)
	{
	case BLOCK_MODE_IDLE:
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void SptColBlock (Sprite *spt, Sprite *obs, u32 dir, Map *map)
{
	if (obs == NULL)	// Map collision
	{
	}
	else	// Sprite collision
	{
		if (obs->type == SPRITE_TYPE_OIRAM)
		{
			if ((dir == SPRITE_COL_DIR_DOWN) && 
				(spt->vars.block.state == BLOCK_IDLE_STATE_FULL))
			{
				SptListFile flowerFile;

				flowerFile.spriteType = SPRITE_TYPE_FLOWER;
				flowerFile.priority = spt->priority + 1;
				ASSERT (spt->priority < SPRITE_PRIORITY_NUM);
				flowerFile.x = spt->x;
				flowerFile.y = spt->y;

				BlockSetMode (spt, BLOCK_MODE_IDLE, BLOCK_IDLE_STATE_EMPTY);
				MapCreateSprite (map, &flowerFile);
			}
		}
		else
		{
		}
	}
}



// ----- Local functions -----

static void BlockSetMode (Sprite *spt, u32 mode, u32 state)
{
	ASSERT (mode < BLOCK_MODE_NUM);

	spt->vars.block.mode	= mode;
	spt->vars.block.state	= state;

	switch (mode)
	{
	case BLOCK_MODE_IDLE:
		switch (state)
		{
		case BLOCK_IDLE_STATE_FULL:
			SptSetAnim (spt, BLOCK_ANIM_FULL, FALSE);
			break;

		case BLOCK_IDLE_STATE_EMPTY:
			SptSetAnim (spt, BLOCK_ANIM_EMPTY, FALSE);
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		break;
	}
}
