#include "Main.h"
#include "Map.h"
#include "MapData.h"
#include "Flower.h"



// ----- Local function prototypes -----

static void FlowerSetMode (Sprite *spt, u32 mode, u32 state);



// ----- Global functions -----

void SptInitFlower (Sprite *spt, Map *map)
{
	FlowerSetMode (spt, FLOWER_MODE_IDLE, FLOWER_IDLE_STATE_GROW);
}

void SptUpdateFlower (Sprite *spt, Map *map)
{
	switch (spt->vars.flower.mode)
	{
	case FLOWER_MODE_IDLE:
		switch (spt->vars.flower.state)
		{
		case FLOWER_IDLE_STATE_GROW:
			if (--spt->vars.flower.timer == 0)
			{
				FlowerSetMode (spt, FLOWER_MODE_IDLE, FLOWER_IDLE_STATE_IDLE);
			}
			break;

		case FLOWER_IDLE_STATE_IDLE:
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void SptColFlower (Sprite *spt, Sprite *obs, u32 dir, Map *map)
{
	if (obs == NULL)	// Map collision
	{
	}
	else	// Sprite collision
	{
		if (obs->type == SPRITE_TYPE_OIRAM)
		{
			spt->flags &= ~SPRITE_FLAG_ALIVE;
		}
		else
		{
		}
	}
}



// ----- Local functions -----

static void FlowerSetMode (Sprite *spt, u32 mode, u32 state)
{
	ASSERT (mode < FLOWER_MODE_NUM);

	spt->vars.flower.mode	= mode;
	spt->vars.flower.state	= state;

	switch (mode)
	{
	case FLOWER_MODE_IDLE:
		switch (state)
		{
		case FLOWER_IDLE_STATE_GROW:
			spt->yVel = FLOWER_GROW_SPEED;
			spt->vars.flower.timer = SPT_FX(-TILE_SIZE) / FLOWER_GROW_SPEED;
			break;

		case FLOWER_IDLE_STATE_IDLE:
			spt->yVel = 0;
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		break;
	}
}
