// ----- Change log -----
// 02/26/05 - Created
// ----------------------



// ----- Headers -----

#include "Main.h"
#include "Font.h"



static const u16 dFontBmp[64][16];

// ----- Local function prototypes -----

static void DrawChar8(Image *screen, s32 x, s32 y, const u16 *font, u32 colors);
static void DrawChar16(Image *screen, s32 x, s32 y, const u16 *font, u32 colors);



// ----- Global functions -----

void FontPrint8 (Image *screen, s32 x, s32 y, const char *text, u32 colors)
{
	s32 curX = x;

	while (*text)
	{
		if (*text == '\n')
		{
			curX = x;
			y += 12;
		}
		else
		{
			DrawChar8 (screen, curX, y, dFontBmp[*text - ' '], colors);
			curX += 8;
		}
		text++;
	}
}

void FontPrint16 (Image *screen, s32 x, s32 y, const char *text, u32 colors)
{
	s32 curX = x;

	while (*text)
	{
		if (*text == '\n')
		{
			curX = x;
			y += 16;
		}
		else
			DrawChar16 (screen, curX, y, dFontBmp[*text - ' '], colors);

		curX += 16;
		text++;
	}
}



// ----- Local functions -----

static void DrawChar8(Image *screen, s32 x, s32 y, const u16 *font, u32 colors)
{
	u8 *dest;
	u16 srcPx, oldSrcPx = 0;
	s32 i, ly;
	u8 c1, c2, c3;
	c1 = (u8) (colors & 0xf);
	c2 = (u8) ((colors >> 8) & 0xf);
	c3 = (u8) ((colors >> 16) & 0xf);
	dest = (screen->pixels + y*screen->pitch + x);

	for(ly = 0; ly < 9; ly++)
	{
		if (ly == 8)
			srcPx = 0;
		else
			srcPx = font[ly*2];

		for(i = 0; i < 9; i++)
		{
			if(srcPx & (32768>>i*2)) dest[i] = c1;
			else if(oldSrcPx & (131072>>i*2)) dest[i] = c2;
			else if (c3 != 0) dest[i] = c3;
		}

		oldSrcPx = srcPx;

		dest += screen->pitch;
	}
}

static void DrawChar16(Image *screen, s32 x, s32 y, const u16 *font, u32 colors)
{
	u8 *dest;
	u16 srcPx, oldSrcPx = 0;
	s32 i, ly;
	u8 c1, c2, c3;
	c1 = (u8) (colors & 0xf);
	c2 = (u8) ((colors >> 8) & 0xf);
	c3 = (u8) ((colors >> 16) & 0xf);
	dest = (screen->pixels + y*screen->pitch + x);

	for(ly = 0; ly < 17; ly++)
	{
		if (ly == 16)
			srcPx = 0;
		else
			srcPx = font[ly];


		for(i = 0; i < 17; i++)
		{
			if(srcPx & (32768>>i)) dest[i] = c1;
			else if(oldSrcPx & (65536>>i)) dest[i] = c2;
			else if (c3 != 0) dest[i] = c3;
		}

		oldSrcPx = srcPx;

		dest += screen->pitch;
	}
}



// ----- Data -----

static const u16 dFontBmp[64][16] = {
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 
		0x1800, 0x1800, 0x1800, 0x0, 0x0, 0x1800, 0x1800, 0x0}, 
	{
		0xe70, 0xe70, 0x660, 0x0, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x630, 0x630, 0x1ffc, 0x1ffc, 0x630, 0x630, 
		0x630, 0x1ffc, 0x1ffc, 0x630, 0x630, 0x0, 0x0, 0x0}, 
	{
		0x180, 0x180, 0x7e0, 0xff0, 0xdb0, 0xd80, 0x780, 0x3e0, 
		0x1f0, 0x1b0, 0xdb0, 0xff0, 0x7e0, 0x180, 0x180, 0x0}, 
	{
		0x0, 0x0, 0x8, 0x1c18, 0x1430, 0x1c60, 0xc0, 0x180, 
		0x300, 0x600, 0xc70, 0x1850, 0x3070, 0x2000, 0x0, 0x0}, 
	{
		0x0, 0xf80, 0x1cc0, 0x1840, 0x18c0, 0xd80, 0x700, 0xf00, 
		0x1980, 0x30d0, 0x3070, 0x3060, 0x3ff0, 0x1fb0, 0x0, 0x0}, 
	{
		0x700, 0x700, 0x600, 0x600, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x18, 0x38, 0x70, 0x60, 0xe0, 0xc0, 0xc0, 0xc0, 
		0xc0, 0xc0, 0xe0, 0x60, 0x70, 0x38, 0x18, 0x0}, 
	{
		0x1800, 0x1c00, 0xe00, 0x600, 0x700, 0x300, 0x300, 0x300, 
		0x300, 0x300, 0x700, 0x600, 0xe00, 0x1c00, 0x1800, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x5a0, 0x7e0, 0x3c0, 
		0x7e0, 0x5a0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x180, 0x180, 0x180, 0x180, 0x1ff8, 
		0x1ff8, 0x180, 0x180, 0x180, 0x180, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x600, 0x600, 0xe00, 0xc00, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff8, 
		0xff8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0x600, 0x600, 0x0}, 
	{
		0x0, 0x0, 0xc, 0x1c, 0x38, 0x70, 0xe0, 0x1c0, 
		0x380, 0x700, 0xe00, 0x1c00, 0x3800, 0x3000, 0x0, 0x0}, 
	{
		0x7e0, 0xff0, 0x1c38, 0x1818, 0x381c, 0x300c, 0x300c, 0x300c, 
		0x300c, 0x300c, 0x381c, 0x1818, 0x1c38, 0xff0, 0x7e0, 0x0}, 
	{
		0x180, 0x380, 0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 
		0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 0x3c0, 0x0}, 
	{
		0x7e0, 0xff0, 0x1c38, 0x1818, 0x18, 0x18, 0x38, 0x70, 
		0xe0, 0x1c0, 0x380, 0x700, 0xe00, 0x1ffc, 0x1ffc, 0x0}, 
	{
		0x7f0, 0xff8, 0x1c1c, 0x180c, 0xc, 0x1c, 0x78, 0x78, 
		0x1c, 0xc, 0xc, 0x180c, 0x1c3c, 0xff8, 0x7e0, 0x0}, 
	{
		0x30, 0x70, 0xf0, 0x1f0, 0x3b0, 0x730, 0xe30, 0x1c30, 
		0x3830, 0x3ffc, 0x3ffc, 0x30, 0x30, 0x30, 0x30, 0x0}, 
	{
		0x1ff8, 0x1ff8, 0x1800, 0x1800, 0x1800, 0x1fe0, 0x1ff0, 0x38, 
		0x18, 0x18, 0x18, 0x18, 0x1838, 0x1ff0, 0xfe0, 0x0}, 
	{
		0x78, 0x1f8, 0x3c0, 0x700, 0xe00, 0xc00, 0x1fe0, 0x1ff0, 
		0x3838, 0x3018, 0x3018, 0x3018, 0x3838, 0x1ff0, 0xfe0, 0x0}, 
	{
		0x1ffc, 0x1ffc, 0x100c, 0x1c, 0x38, 0x70, 0xe0, 0x1c0, 
		0x180, 0x380, 0x300, 0x700, 0x600, 0x600, 0x600, 0x0}, 
	{
		0x7e0, 0xff0, 0x1c38, 0x1818, 0x1818, 0x1c38, 0xff0, 0xff0, 
		0x1c38, 0x3818, 0x3018, 0x3018, 0x3838, 0x1ff0, 0xfe0, 0x0}, 
	{
		0x7f0, 0xff8, 0x1c1c, 0x180c, 0x180c, 0x180c, 0x1c1c, 0xffc, 
		0x7fc, 0x18, 0x38, 0x70, 0xe0, 0x3c0, 0x380, 0x0}, 
	{
		0x0, 0x0, 0x1800, 0x1800, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x1800, 0x1800, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0xc00, 0xc00, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0xc00, 0xc00, 0xc00, 0x800, 0x0, 0x0}, 
	{
		0x0, 0x30, 0x70, 0xe0, 0x1c0, 0x380, 0x700, 0xe00, 
		0xe00, 0x700, 0x380, 0x1c0, 0xe0, 0x70, 0x30, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0xff0, 0xff0, 0x0, 0x0, 
		0x0, 0xff0, 0xff0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0xc00, 0xe00, 0x700, 0x380, 0x1c0, 0xe0, 0x70, 
		0x70, 0xe0, 0x1c0, 0x380, 0x700, 0xe00, 0xc00, 0x0}, 
	{
		0xff0, 0x1ff8, 0x3838, 0x3018, 0x3018, 0x3838, 0x1870, 0xe0, 
		0x1c0, 0x180, 0x180, 0x0, 0x0, 0x180, 0x180, 0x0}, 
	{
		0x7e0, 0x1ff8, 0x383c, 0x300e, 0x63c6, 0x67e6, 0x6666, 0x6666, 
		0x6666, 0x6766, 0x63ee, 0x71fc, 0x3878, 0x1800, 0x0, 0x0}, 
	{
		0x180, 0x180, 0x3c0, 0x3c0, 0x7e0, 0x660, 0xe70, 0xc30, 
		0x1c38, 0x1ff8, 0x3ffc, 0x381c, 0x700e, 0x700e, 0xf81f, 0x0}, 
	{
		0x7f80, 0x3fe0, 0x3070, 0x3030, 0x3030, 0x3070, 0x3fe0, 0x3ff0, 
		0x3078, 0x301c, 0x300c, 0x300c, 0x301c, 0x3ff8, 0x7ff0, 0x0}, 
	{
		0x7e0, 0xff0, 0x1c38, 0x381c, 0x300c, 0x7000, 0x6000, 0x6000, 
		0x6000, 0x6000, 0x7006, 0x300e, 0x383c, 0x1ff8, 0xfe0, 0x0}, 
	{
		0x7fe0, 0x3ff0, 0x3078, 0x3018, 0x301c, 0x300c, 0x300c, 0x300c, 
		0x300c, 0x300c, 0x301c, 0x3018, 0x3078, 0x3ff0, 0x7fe0, 0x0}, 
	{
		0x7ff8, 0x3ff8, 0x3008, 0x3000, 0x3000, 0x3080, 0x3f80, 0x3f80, 
		0x3080, 0x3000, 0x3000, 0x3000, 0x3008, 0x3ff8, 0x7ff8, 0x0}, 
	{
		0x7ff8, 0x3ff8, 0x3008, 0x3000, 0x3000, 0x3080, 0x3f80, 0x3f80, 
		0x3080, 0x3000, 0x3000, 0x3000, 0x3000, 0x3000, 0x7800, 0x0}, 
	{
		0xfc0, 0x1fe0, 0x3870, 0x7038, 0x6018, 0xe000, 0xc000, 0xc000, 
		0xc03e, 0xc03e, 0xe018, 0x6018, 0x7038, 0x3ff0, 0x1fe0, 0x0}, 
	{
		0x781e, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x3ffc, 0x3ffc, 
		0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x781e, 0x0}, 
	{
		0x3c0, 0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 
		0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 0x3c0, 0x0}, 
	{
		0x78, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 
		0x7830, 0x3030, 0x3030, 0x3030, 0x3870, 0x1fe0, 0xfc0, 0x0}, 
	{
		0x7878, 0x3070, 0x30e0, 0x31c0, 0x3380, 0x3700, 0x3e00, 0x3e00, 
		0x3700, 0x3380, 0x31c0, 0x30e0, 0x3070, 0x3038, 0x783c, 0x0}, 
	{
		0x3c00, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 
		0x1800, 0x1800, 0x1800, 0x1800, 0x1804, 0x1ffc, 0x3ffc, 0x0}, 
	{
		0x700e, 0x381c, 0x3c3c, 0x3e7c, 0x37ec, 0x33cc, 0x318c, 0x300c, 
		0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x781e, 0x0}, 
	{
		0x783c, 0x3c18, 0x3c18, 0x3618, 0x3618, 0x3318, 0x3318, 0x3198, 
		0x3198, 0x30d8, 0x30d8, 0x3078, 0x3078, 0x3038, 0x783c, 0x0}, 
	{
		0x7e0, 0xff0, 0x1c38, 0x381c, 0x300c, 0x700e, 0x6006, 0x6006, 
		0x6006, 0x700e, 0x300c, 0x381c, 0x1c38, 0xff0, 0x7e0, 0x0}, 
	{
		0x3ff0, 0x1ff8, 0x181c, 0x180c, 0x180c, 0x180c, 0x181c, 0x1ff8, 
		0x1ff0, 0x1800, 0x1800, 0x1800, 0x1800, 0x1800, 0x3c00, 0x0}, 
	{
		0xfc0, 0x1fe0, 0x3870, 0x7038, 0x6018, 0xe01c, 0xc00c, 0xc00c, 
		0xc00c, 0xe07c, 0x6078, 0x7038, 0x387c, 0x1fee, 0xfc6, 0x0}, 
	{
		0x3ff0, 0x1ff8, 0x181c, 0x180c, 0x180c, 0x180c, 0x181c, 0x1ff8, 
		0x1ff0, 0x1830, 0x1838, 0x1818, 0x181c, 0x180c, 0x3c0e, 0x0}, 
	{
		0x7f8, 0xffc, 0x1c0e, 0x1806, 0x1806, 0x1c00, 0xfe0, 0x7f8, 
		0x3c, 0xe, 0x3006, 0x3006, 0x380e, 0x1ffc, 0xff8, 0x0}, 
	{
		0x7ffe, 0x7ffe, 0x4182, 0x180, 0x180, 0x180, 0x180, 0x180, 
		0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 0x3c0, 0x0}, 
	{
		0x781e, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 
		0x300c, 0x300c, 0x381c, 0x1818, 0x1c38, 0xff0, 0x7e0, 0x0}, 
	{
		0x781e, 0x300c, 0x300c, 0x1818, 0x1818, 0x1818, 0xc30, 0xc30, 
		0xc30, 0x660, 0x660, 0x660, 0x3c0, 0x3c0, 0x180, 0x0}, 
	{
		0x781e, 0x300c, 0x300c, 0x300c, 0x300c, 0x300c, 0x318c, 0x318c, 
		0x318c, 0x399c, 0x1998, 0x1bd8, 0x1ff8, 0xe70, 0xc30, 0x0}, 
	{
		0x781e, 0x300c, 0x381c, 0x1818, 0x1c38, 0xe70, 0x7e0, 0x3c0, 
		0x7e0, 0xe70, 0x1c38, 0x1818, 0x381c, 0x300c, 0x781e, 0x0}, 
	{
		0x781e, 0x300c, 0x381c, 0x1818, 0x1c38, 0xe70, 0x7e0, 0x3c0, 
		0x180, 0x180, 0x180, 0x180, 0x180, 0x180, 0x3c0, 0x0}, 
	{
		0x1ffc, 0x1ffc, 0x100c, 0x1c, 0x38, 0x70, 0xe0, 0x1c0, 
		0x380, 0x700, 0xe00, 0x1c00, 0x3804, 0x3ffc, 0x3ffc, 0x0}, 
	{
		0x0, 0x7c, 0x78, 0x60, 0x60, 0x60, 0x60, 0x60, 
		0x60, 0x60, 0x60, 0x60, 0x78, 0x7c, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x3000, 0x3800, 0x1c00, 0xe00, 0x700, 0x380, 
		0x1c0, 0xe0, 0x70, 0x38, 0x1c, 0xc, 0x0, 0x0}, 
	{
		0x0, 0x3e00, 0x1e00, 0x600, 0x600, 0x600, 0x600, 0x600, 
		0x600, 0x600, 0x600, 0x600, 0x1e00, 0x3e00, 0x0, 0x0}, 
	{
		0x100, 0x380, 0x7c0, 0xee0, 0xc60, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0}, 
	{
		0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
		0x0, 0x0, 0x0, 0x0, 0x0, 0xff8, 0xff8, 0x0}
};
