// ----- Change log -----
// 02-26-05	Created



// ----- Headers -----

#include "Main.h"
#include "Font.h"
#include "Game.h"
#include "Image.h"
#include "Input.h"
#include "Map.h"
#include "MapData.h"



// ----- Global variables -----

u32 thisTick, lastTick;
s32 lastFrameRetrace;				// To limit frame rate
s32 oldRetraceCount, frameCounter;	// For FPS calculating
BOOL bActive, gQuitGame;
char fpsStr[16];

Map gMap;



// ----- Local function prototypes -----

static BOOL Init ();
static void Cleanup();
static void UpdateFrame ();



// ----- Global functions -----

int main (void)
{
	Init ();

	while (!gQuitGame)
	{
		if (bActive)
		{
			UpdateFrame ();
		}
	}

	Cleanup ();

	return 0;
}

END_OF_MAIN();



// ----- Local functions -----

static BOOL Init ()
{
	allegro_init ();
	install_timer ();
	install_keyboard ();

	ImgCreate (&gScreen, 640, 480, 256);

	if (set_gfx_mode(GFX_AUTODETECT, gScreen.width, gScreen.height, 0, 0) != 0)
	{
		ERROR_TEXT("Couldn't set graphic mode!");
	}

	memClear (&gGame, sizeof(Game));
	memClear (fpsStr, 16);
	lastFrameRetrace = retrace_count;
	oldRetraceCount = retrace_count;
	gQuitGame = FALSE;
	bActive = TRUE;
	gGameSwitch = 0;

	GameSetMode (&gGame, GAME_MODE_TITLE, 0);

	return TRUE;
}

static void Cleanup ()
{
	GameSetMode (&gGame, GAME_MODE_NONE, 0);	// Clean up whatever state it was in
	ImgFree (&gScreen);
	allegro_exit ();
}

static void UpdateFrame ()
{
	s32 i, j;

	if (retrace_count >= lastFrameRetrace + 2)
	{
		lastFrameRetrace = retrace_count;

		// --- Update engine modules ---

		memClear (gScreen.pixels, gScreen.pitch*gScreen.height);

		InputUpdate ();

		if (++frameCounter >= 32)
		{
			s32 newRetraceCount = retrace_count;
			s32 fps;
			s32 pos;

			frameCounter = 0;

			fps = 32*70 * 10 / (newRetraceCount-oldRetraceCount);
			oldRetraceCount = newRetraceCount;

			strcpy (fpsStr, "FPS: ");
			itoa (fps/10, fpsStr+5, 10);
			pos = strlen(fpsStr);
			fpsStr[pos+5] = '.';
			itoa (fps%10, fpsStr+pos+5+1, 10);
		}



		// --- Update game ---

		GameUpdate (&gGame);



		// --- Display the screen ---

		//FontPrint8 (&gScreen, 2, 2, fpsStr, FONT_COLORS(4, 254, 0));

		set_palette (gScreen.palette);

		acquire_screen ();
		for (j = 0; j < gScreen.height; j++)
		{
			for (i = 0; i < gScreen.width; i++)
			{
				putpixel (screen, i, j, gScreen.pixels[j*gScreen.pitch+i]);
			}
		}
		release_screen ();

	}	// end retrace != last frame's retrace
}
