#include "Main.h"
#include "Map.h"
#include "MapData.h"
#include "Oiram.h"



// ----- Data -----

// --- Tileset collisions ---

const u8 dTile1Collision[] =
{
	COLLISION_NONE,		// Blank
	COLLISION_NONE,		// Sky
	COLLISION_NONE,		// Cloud
	COLLISION_TOP,		// Grass mid
	COLLISION_NONE,		// Dirt mid
	COLLISION_TOP|COLLISION_LEFT,	// Grass edge - left
	COLLISION_TOP|COLLISION_RIGHT,	// Grass edge - right
	COLLISION_LEFT,		// Dirt edge - left
	COLLISION_RIGHT,	// Dirt edge - right
	COLLISION_TOP,		// Grass underground edge - left
	COLLISION_TOP,		// Grass underground edge - right
	COLLISION_SOLID|COLLISION_HAZARD,	// Blank - hazard
	COLLISION_SOLID,	// Blank - solid
	COLLISION_SOLID|COLLISION_PIPE,	// Pipe - top
	COLLISION_SOLID,	// Pipe - bottom
	COLLISION_SOLID,	// Pipe - top 2
};

const u8 dTileCastleCollision[] =
{
	COLLISION_NONE,		// Blank
	COLLISION_TOP,		// Ground
	COLLISION_NONE,		// 
	COLLISION_TOP|COLLISION_LEFT,		// 
	COLLISION_TOP|COLLISION_RIGHT,		// 
	COLLISION_LEFT,		// 
	COLLISION_RIGHT,		// 
	COLLISION_BOTTOM,		// 
	COLLISION_BOTTOM|COLLISION_LEFT,		// 
	COLLISION_BOTTOM|COLLISION_RIGHT,		// 
	COLLISION_TOP|COLLISION_HAZARD,		// Lava top
	COLLISION_NONE,		// Lava
	COLLISION_SOLID,	// Pipe - top
	COLLISION_SOLID,	// Pipe - mid
	COLLISION_SOLID,	// Pipe - top 2
	COLLISION_SOLID,	// Blank - solid
	COLLISION_NONE,	// Blank - solid
	COLLISION_LEFT|COLLISION_TEXT,	// Peach
	COLLISION_LEFT|COLLISION_TEXT,	// Peach
	COLLISION_LEFT|COLLISION_TEXT,	// Peach
	COLLISION_LEFT|COLLISION_TEXT,	// Peach
};



// --- Sprite anims ---

// - Oiram -

const SptAnim dAnimOiramIdle =
{
	1,	// Frame count
	{
		{ 0, 0 },
	}
};

const SptAnim dAnimOiramWalk =
{
	2,	// Frame count
	{
		{ 2, 7 },
		{ 1, 7 },
	}
};

const SptAnim dAnimOiramRun =
{
	2,	// Frame count
	{
		{ 2, 3 },
		{ 1, 3 },
	}
};

const SptAnim dAnimOiramJump =
{
	1,	// Frame count
	{
		{ 1, 0 },
	}
};

const SptAnim dAnimOiramFall =
{
	1,	// Frame count
	{
		{ 1, 0 },
	}
};

const SptAnim dAnimOiramDie =
{
	1,	// Frame count
	{
		{ 3, 0 },
	}
};

const SptAnim dAnimOiramPipe =
{
	1,	// Frame count
	{
		{ 3, 0 },
	}
};

const SptAnim *dAnimListOiram[OIRAM_ANIM_NUM] =
{
	&dAnimOiramIdle,			// OIRAM_ANIM_IDLE
	&dAnimOiramWalk,			// OIRAM_ANIM_WALK
	&dAnimOiramRun,				// OIRAM_ANIM_RUN
	&dAnimOiramJump,			// OIRAM_ANIM_JUMP
	&dAnimOiramFall,			// OIRAM_ANIM_FALL
	&dAnimOiramDie,				// OIRAM_ANIM_DIE
	&dAnimOiramPipe,			// OIRAM_ANIM_PIPE
};



// - Turtle -

const SptAnim dAnimTurtleWalk =
{
	2,	// Frame count
	{
		{ 0, 8 },
		{ 1, 8 },
	}
};

const SptAnim dAnimTurtleDie =
{
	1,	// Frame count
	{
		{ 2, 0 },
	}
};

const SptAnim *dAnimListTurtle[TURTLE_ANIM_NUM] =
{
	&dAnimTurtleWalk,			// TURTLE_ANIM_WALK
	&dAnimTurtleDie,			// TURTLE_ANIM_DIE
};



// - Block -

const SptAnim dAnimBlockFull =
{
	1,	// Frame count
	{
		{ 0, 0 },
	}
};

const SptAnim dAnimBlockEmpty =
{
	1,	// Frame count
	{
		{ 1, 0 },
	}
};

const SptAnim *dAnimListBlock[BLOCK_ANIM_NUM] =
{
	&dAnimBlockFull,			// BLOCK_ANIM_FULL
	&dAnimBlockEmpty,			// BLOCK_ANIM_EMPTY
};



// - Flower -

const SptAnim dAnimFlowerIdle =
{
	2,	// Frame count
	{
		{ 0, 8 },
		{ 1, 8 },
	}
};

const SptAnim *dAnimListFlower[FLOWER_ANIM_NUM] =
{
	&dAnimFlowerIdle,			// FLOWER_ANIM_IDLE
};



// - Sine -

const SptAnim dAnimSineIdle =
{
	1,	// Frame count
	{
		{ 0, 0 },
	}
};

const SptAnim *dAnimListSine[SINE_ANIM_NUM] =
{
	&dAnimSineIdle,			// SINE_ANIM_IDLE
};



// --- Sprite types ---

const SptTypeFile dSptType[SPRITE_TYPE_NUM] =
{
	{	// SPRITE_TYPE_OIRAM
		SPT_FX(16), SPT_FX(64),	// Hotspot
		32, 64,					// Dimensions
		SptInitOiram,			// Init function
		SptUpdateOiram,			// Update function
		SptRenderOiram,			// Render function
		SptColOiram,			// Collision function
		{ -12, -40, 24, 40 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "Oiram.bmp" },		// Image file
		dAnimListOiram,			// Anim list
	},
	{	// SPRITE_TYPE_TURTLE
		SPT_FX(16), SPT_FX(32),	// Hotspot
		32, 32,					// Dimensions
		SptInitTurtle,			// Init function
		SptUpdateTurtle,		// Update function
		SptRenderImg,			// Render function
		SptColTurtle,			// Collision function
		{ -11, -10, 23, 10 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "Turtle.bmp" },		// Image file
		dAnimListTurtle,		// Anim list
	},
	{	// SPRITE_TYPE_PIRANHA
		SPT_FX(16), SPT_FX(32),	// Hotspot
		32, 32,					// Dimensions
		SptInitTurtle,			// Init function
		SptUpdateTurtle,		// Update function
		SptRenderImg,			// Render function
		SptColTurtle,			// Collision function
		{ -12, -14, 22, 14 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "Piranha.bmp" },		// Image file
		dAnimListTurtle,		// Anim list
	},
	{	// SPRITE_TYPE_TROLL
		SPT_FX(128), SPT_FX(128),	// Hotspot
		256, 256,					// Dimensions
		SptInitTurtle,			// Init function
		SptUpdateTurtle,		// Update function
		SptRenderImg,			// Render function
		SptColTurtle,			// Collision function
		{ -64, -78, 128, 206 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "Troll.bmp" },		// Image file
		dAnimListTurtle,		// Anim list
	},
	{	// SPRITE_TYPE_BLOCK
		SPT_FX(16), SPT_FX(32),	// Hotspot
		32, 32,					// Dimensions
		SptInitBlock,			// Init function
		SptUpdateBlock,			// Update function
		SptRenderImg,			// Render function
		SptColBlock,			// Collision function
		{ -16, -32, 32, 32 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "Block.bmp" },		// Image file
		dAnimListBlock,			// Anim list
	},
	{	// SPRITE_TYPE_FLOWER
		SPT_FX(16), SPT_FX(32),	// Hotspot
		32, 32,					// Dimensions
		SptInitFlower,			// Init function
		SptUpdateFlower,		// Update function
		SptRenderImg,			// Render function
		SptColFlower,			// Collision function
		{ -16, -32, 32, 32 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "SineFlower.bmp" },	// Image file
		dAnimListFlower,		// Anim list
	},
	{	// SPRITE_TYPE_SINE
		SPT_FX(32), SPT_FX(16),	// Hotspot
		64, 16,					// Dimensions
		SptInitSine,			// Init function
		SptUpdateSine,			// Update function
		SptRenderSine,			// Render function
		SptColSine,				// Collision function
		{ -32, -16, 64, 32 },	// Collision box
		SPRITE_FLAG_COLLIDE,	// Flags
		{ "" },					// Image file (dynamic for this sprite)
		dAnimListSine,			// Anim list
	},
};



// --- Levels ---

const SptListFile dMapFirstSprites[] =
{
	{ SPRITE_TYPE_OIRAM, SPRITE_PRIORITY_1, SPT_FX(2*TILE_SIZE+16), SPT_FX(21*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(6*TILE_SIZE), SPT_FX(20*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(41*TILE_SIZE), SPT_FX(21*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(45*TILE_SIZE), SPT_FX(21*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(47*TILE_SIZE), SPT_FX(21*TILE_SIZE) }, 
	{ SPRITE_TYPE_PIRANHA, SPRITE_PRIORITY_1, SPT_FX(23*TILE_SIZE), SPT_FX(21*TILE_SIZE) }, 
	{ SPRITE_TYPE_BLOCK, SPRITE_PRIORITY_1, SPT_FX(19*TILE_SIZE)/2, SPT_FX(15*TILE_SIZE) }, 
};

const SptListFile dMapCastleSprites[] =
{
	{ SPRITE_TYPE_OIRAM, SPRITE_PRIORITY_1, SPT_FX(3*TILE_SIZE+16), SPT_FX(28*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(15*TILE_SIZE+16), SPT_FX(23*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(27*TILE_SIZE+16), SPT_FX(25*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(76*TILE_SIZE+16), SPT_FX(29*TILE_SIZE) }, 
	{ SPRITE_TYPE_TURTLE, SPRITE_PRIORITY_1, SPT_FX(84*TILE_SIZE+16), SPT_FX(24*TILE_SIZE) }, 
	{ SPRITE_TYPE_PIRANHA, SPRITE_PRIORITY_1, SPT_FX(36*TILE_SIZE+16), SPT_FX(28*TILE_SIZE) }, 
	{ SPRITE_TYPE_PIRANHA, SPRITE_PRIORITY_1, SPT_FX(38*TILE_SIZE+16), SPT_FX(28*TILE_SIZE) }, 
	{ SPRITE_TYPE_PIRANHA, SPRITE_PRIORITY_1, SPT_FX(39*TILE_SIZE+16), SPT_FX(28*TILE_SIZE) }, 
	{ SPRITE_TYPE_PIRANHA, SPRITE_PRIORITY_1, SPT_FX(41*TILE_SIZE+16), SPT_FX(28*TILE_SIZE) }, 
	{ SPRITE_TYPE_BLOCK, SPRITE_PRIORITY_1, SPT_FX(67*TILE_SIZE+16), SPT_FX(11*TILE_SIZE) }, 
	{ SPRITE_TYPE_TROLL, SPRITE_PRIORITY_1, SPT_FX(104*TILE_SIZE+16), SPT_FX(25*TILE_SIZE) }, 
};

const MapFile dMap[LEVEL_ID_NUM] =
{
	{	// LEVEL_ID_FIRST
		{ "Map1.bmp" },		// Map layout bitmap
		{ "Map1Sky.bmp" },	// Map BG layout bitmap
		{ "Tile1.bmp" },	// Tileset bitmap
		32, 32,				// Min scroll values
		63*TILE_SIZE, 23*TILE_SIZE,	// Max scroll values
		128, 128,			// BG scroll multipliers
		0, 0,				// BG scroll offset
		7,					// Sprite count
		0,					// Follow sprite (0xFF for none)
		dMapFirstSprites,	// Sprite list
		dTile1Collision,	// Tile collision
	},
	{	// LEVEL_ID_CASTLE
		{ "MapCastle.bmp" },		// Map layout bitmap
		{ "MapCastleBg.bmp" },	// Map BG layout bitmap
		{ "TileCastle.bmp" },	// Tileset bitmap
		0, 32,				// Min scroll values
		127*TILE_SIZE, 31*TILE_SIZE,	// Max scroll values
		128, 256,			// BG scroll multipliers
		SPT_FX(1)/3, 0,				// BG scroll offset
		11,					// Sprite count
		0,					// Follow sprite (0xFF for none)
		dMapCastleSprites,	// Sprite list
		dTileCastleCollision,	// Tile collision
	},
};
