#include "Main.h"
#include "Input.h"
#include "Font.h"
#include "Game.h"
#include "Map.h"
#include "MapData.h"
#include "Oiram.h"



// ----- Local function prototypes -----

static void OiramSetMode (Sprite *spt, u32 mode, u32 state, Map *map);
static BOOL OiramGroundSnap (Sprite *spt, Map *map);


RGB dOiramOverallsNormal = { 74>>2, 108>>2, 166>>2 };
RGB dOiramOverallsFlower = { 191>>2, 69>>2, 34>>2 };


// ----- Global functions -----

void SptInitOiram (Sprite *spt, Map *map)
{
	OiramSetMode (spt, OIRAM_MODE_PIPE, OIRAM_PIPE_STATE_EXIT, map);
}

void SptUpdateOiram (Sprite *spt, Map *map)
{
	switch (spt->vars.oiram.mode)
	{
	case OIRAM_MODE_GROUND:
		switch (spt->vars.oiram.state)
		{
		case OIRAM_GROUND_STATE_IDLE:
		case OIRAM_GROUND_STATE_WALK:
			if (KEYDOWN(KEY_LEFT))
			{
				spt->flags |= SPRITE_FLAG_HFLIP;
				spt->xVel = -OIRAM_WALK_SPEED;
				SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);

				if (KEYDOWN(KEY_RUN))
					spt->vars.oiram.state = OIRAM_GROUND_STATE_RUN;
				else
					spt->vars.oiram.state = OIRAM_GROUND_STATE_WALK;
			}
			else if (KEYDOWN(KEY_RIGHT))
			{
				spt->flags &= ~SPRITE_FLAG_HFLIP;
				spt->xVel = OIRAM_WALK_SPEED;
				SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
				spt->vars.oiram.state = OIRAM_GROUND_STATE_WALK;

				if (KEYDOWN(KEY_RUN))
					spt->vars.oiram.state = OIRAM_GROUND_STATE_RUN;
				else
					spt->vars.oiram.state = OIRAM_GROUND_STATE_WALK;
			}
			else
			{
				spt->xVel = 0;
				SptSetAnim (spt, OIRAM_ANIM_IDLE, FALSE);
				spt->vars.oiram.state = OIRAM_GROUND_STATE_IDLE;
			}
			break;

		case OIRAM_GROUND_STATE_RUN:
			if (KEYDOWN(KEY_RUN))
			{
				if (KEYDOWN(KEY_LEFT))
				{
					spt->flags |= SPRITE_FLAG_HFLIP;
					if (spt->xVel > -OIRAM_RUN_SPEED)
					{
						spt->xVel -= OIRAM_RUN_ACCEL;
						if (spt->xVel < -OIRAM_RUN_SPEED)
							spt->xVel = -OIRAM_RUN_SPEED;
					}
					SptSetAnim (spt, OIRAM_ANIM_RUN, FALSE);
				}
				else if (KEYDOWN(KEY_RIGHT))
				{
					spt->flags &= ~SPRITE_FLAG_HFLIP;
					if (spt->xVel < OIRAM_RUN_SPEED)
					{
						spt->xVel += OIRAM_RUN_ACCEL;
						if (spt->xVel > OIRAM_RUN_SPEED)
							spt->xVel = OIRAM_RUN_SPEED;
					}
					SptSetAnim (spt, OIRAM_ANIM_RUN, FALSE);
				}
				else
				{
					spt->xVel = 0;
					SptSetAnim (spt, OIRAM_ANIM_IDLE, FALSE);
					spt->vars.oiram.state = OIRAM_GROUND_STATE_IDLE;
				}
			}
			else	// Let go of run key
			{
				spt->vars.oiram.state = OIRAM_GROUND_STATE_WALK;
			}
			break;

		default:
			ASSERT_ALWAYS();
		}

		if (OiramGroundSnap (spt, map) == TRUE)
		{
			if (KEYTRIG(KEY_JUMP))
			{
				OiramSetMode (spt, OIRAM_MODE_JUMP, OIRAM_JUMP_STATE_JUMP, map);
			}
			else if (KEYTRIG(KEY_DOWN))
			{
				OiramSetMode (spt, OIRAM_MODE_PIPE, OIRAM_PIPE_STATE_ENTER, map);
			}
		}

		break;

	case OIRAM_MODE_JUMP:
		spt->yVel = OIRAM_JUMP_VEL;

		if (KEYDOWN(KEY_LEFT))
		{
			spt->flags |= SPRITE_FLAG_HFLIP;
			if (spt->xVel > -OIRAM_WALK_SPEED)
				spt->xVel = -OIRAM_WALK_SPEED;
		}
		else if (KEYDOWN(KEY_RIGHT))
		{
			spt->flags &= ~SPRITE_FLAG_HFLIP;
			if (spt->xVel < OIRAM_WALK_SPEED)
				spt->xVel = OIRAM_WALK_SPEED;
		}
		else
		{
			spt->xVel = 0;
		}

		if (KEYUP(KEY_JUMP) || --spt->vars.oiram.timer == 0)
		{
			OiramSetMode (spt, OIRAM_MODE_FALL, OIRAM_FALL_STATE_FALL, map);
		}

		break;

	case OIRAM_MODE_FALL:
		spt->yVel += SPT_GRAVITY;
		if (spt->yVel > SPT_GRAVITY_MAX)
			spt->yVel = SPT_GRAVITY_MAX;

		if (KEYDOWN(KEY_LEFT))
		{
			spt->flags |= SPRITE_FLAG_HFLIP;
			if (spt->xVel > -OIRAM_WALK_SPEED)
				spt->xVel = -OIRAM_WALK_SPEED;
		}
		else if (KEYDOWN(KEY_RIGHT))
		{
			spt->flags &= ~SPRITE_FLAG_HFLIP;
			if (spt->xVel < OIRAM_WALK_SPEED)
				spt->xVel = OIRAM_WALK_SPEED;
		}
		else
		{
			spt->xVel = 0;
		}

		break;

	case OIRAM_MODE_DIE:
		switch (spt->vars.oiram.state)
		{
		case OIRAM_DIE_STATE_DIE:
			if (--spt->vars.oiram.timer == 0)
			{
				OiramSetMode (spt, OIRAM_MODE_DIE, OIRAM_DIE_STATE_FALL, map);
			}
			break;

		case OIRAM_DIE_STATE_FALL:
			spt->yVel += OIRAM_DIE_FALL_ACCEL;
			if (SPT_FX_TO_INT (spt->y - spt->hotspotY) - map->yScroll >= gScreen.height)
				map->flags |= MAP_FLAG_DIE;
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	case OIRAM_MODE_PIPE:
		switch (spt->vars.oiram.state)
		{
		case OIRAM_PIPE_STATE_ENTER:
			if (--spt->vars.oiram.timer == 0)
			{
				spt->yVel = 0;
				map->flags |= MAP_FLAG_NEXT_LEVEL;
			}
			break;

		case OIRAM_PIPE_STATE_EXIT:
			if (--spt->vars.oiram.timer == 0)
			{
				spt->flags |= SPRITE_FLAG_COLLIDE;
				spt->priority--;
				ASSERT (spt->priority < SPRITE_PRIORITY_NUM);
				OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_IDLE, map);
			}
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		ASSERT_ALWAYS();
	}

	if (spt->vars.oiram.flags & OIRAM_FLAG_FLOWER)
	{
		switch (spt->vars.oiram.mode)
		{
		case OIRAM_MODE_GROUND:
		case OIRAM_MODE_JUMP:
		case OIRAM_MODE_FALL:
			if (KEYTRIG(KEY_SHOOT))
			{
				SptListFile sineFile;
				Sprite *sineSpt;

				sineFile.spriteType = SPRITE_TYPE_SINE;
				sineFile.priority = spt->priority - 1;
				ASSERT (sineFile.priority < SPRITE_PRIORITY_NUM);
				sineFile.x = spt->x;
				sineFile.y = spt->y - SPT_FX(spt->height)/4;
				sineSpt = MapCreateSprite (map, &sineFile);
				if (sineSpt)
				{
					if (spt->flags & SPRITE_FLAG_HFLIP)
					{
						sineSpt->flags |= SPRITE_FLAG_HFLIP;
						sineSpt->xVel = -SINE_SPEED;
						sineSpt->x -= SPT_FX(sineSpt->width)/2;
					}
					else
					{
						sineSpt->xVel = SINE_SPEED;
						sineSpt->x += SPT_FX(sineSpt->width)/2;
					}

					sineSpt->vars.sine.freq = rand () % 12 + 2;
					sineSpt->vars.sine.color = 249;
				}
			}
			break;

		case OIRAM_MODE_DIE:
		case OIRAM_MODE_PIPE:
			break;

		default:
			ASSERT_ALWAYS();
		}
	}

	if (spt->vars.oiram.flags & OIRAM_FLAG_HURT)
	{
		if (--spt->vars.oiram.hurtTimer == 0)
			spt->vars.oiram.flags &= ~OIRAM_FLAG_HURT;
	}
}

void SptRenderOiram (Sprite *spt, Map *map, Image *screen)
{
	BOOL bDraw = TRUE;

	if (spt->vars.oiram.flags & OIRAM_FLAG_HURT)
	{
		if (spt->vars.oiram.hurtTimer & 1)
			bDraw = FALSE;
	}

	if (bDraw == TRUE)
	{
		SptRenderImg (spt, map, screen);
	}
}

void SptColOiram (Sprite *spt, Sprite *obs, u32 dir, Map *map)
{
	if (obs == NULL)	// Map collision
	{
		u8 collisionType = dir & 0xf0;
		dir &= 0xf;

		switch (collisionType)
		{
		case COLLISION_PIPE:
			spt->vars.oiram.flags |= OIRAM_FLAG_ON_PIPE;

		case COLLISION_NONE:
			if (dir == SPRITE_COL_DIR_LEFT || dir == SPRITE_COL_DIR_RIGHT)
				spt->newX = spt->x;
			else if (dir == SPRITE_COL_DIR_UP || dir == SPRITE_COL_DIR_DOWN)
			{
				spt->newY = spt->y;
				spt->yVel = 0;
				if (spt->vars.oiram.mode == OIRAM_MODE_JUMP)
					OiramSetMode (spt, OIRAM_MODE_FALL, OIRAM_FALL_STATE_FALL, map);
			}

			switch (spt->vars.oiram.mode)
			{
			case OIRAM_MODE_FALL:
				if (dir == SPRITE_COL_DIR_DOWN)
				{
					OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_DETECT, map);
					OiramGroundSnap (spt, map);
				}
				break;

			default:
				break;
			}
			break;

		case COLLISION_HAZARD:
			OiramSetMode (spt, OIRAM_MODE_DIE, OIRAM_DIE_STATE_DIE, map);
			break;

		case COLLISION_TEXT:
			MapCreateTextBox (map, "YOU SAVED ME! THANK YOU!");
			map->flags |= MAP_FLAG_END_GAME;
			break;

		default:
			ASSERT_ALWAYS();
		}
	}
	else	// Sprite collision
	{
		if (SPRITE_IS_BADDIE (obs))
		{
			if (dir == SPRITE_COL_DIR_DOWN)
			{
				OiramSetMode (spt, OIRAM_MODE_JUMP, OIRAM_JUMP_STATE_JUMP, map);
			}
			else if (!(spt->vars.oiram.flags & OIRAM_FLAG_HURT))
			{
				if (spt->vars.oiram.flags & OIRAM_FLAG_FLOWER)
				{
					spt->vars.oiram.flags &= ~OIRAM_FLAG_FLOWER;
					spt->vars.oiram.flags |= OIRAM_FLAG_HURT;
					spt->vars.oiram.hurtTimer = OIRAM_HURT_TIME;
					gScreen.palette[OIRAM_PAL_OVERALLS] = dOiramOverallsNormal;
				}
				else
				{
					OiramSetMode (spt, OIRAM_MODE_DIE, OIRAM_DIE_STATE_DIE, map);
				}
			}
		}
		else if (SPRITE_IS_BLOCK(obs))
		{
			if (dir == SPRITE_COL_DIR_UP)
			{
				spt->newY = spt->y;
				spt->yVel = 0;
				if (spt->vars.oiram.mode == OIRAM_MODE_JUMP)
					OiramSetMode (spt, OIRAM_MODE_FALL, OIRAM_FALL_STATE_FALL, map);
			}
			else if (dir == SPRITE_COL_DIR_LEFT || dir == SPRITE_COL_DIR_RIGHT)
			{
				spt->newX = spt->x;
			}
			else if (dir == SPRITE_COL_DIR_DOWN)
			{
				spt->newY = obs->newY + SPT_FX(obs->collision.y);
				OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_DETECT, map);
			}
		}
		else if (SPRITE_IS_FLOWER(obs))
		{
			spt->vars.oiram.flags |= OIRAM_FLAG_FLOWER;
			gScreen.palette[OIRAM_PAL_OVERALLS] = dOiramOverallsFlower;
			if (!(gGameSwitch & GAME_SWITCH_FIRST_FLOWER))
			{
				MapCreateTextBox (map, "YOU GOT A SINE FLOWER!\nNOW YOU CAN SHOOT SINE WAVES\nAT ENEMIES BY PRESSING [ALT]!");
				gGameSwitch |= GAME_SWITCH_FIRST_FLOWER;
			}
		}
	}
}



// ----- Local functions -----

static void OiramSetMode (Sprite *spt, u32 mode, u32 state, Map *map)
{
	ASSERT (mode < OIRAM_MODE_NUM);

	if (mode != spt->vars.oiram.mode)
	{
		spt->vars.oiram.flags &= ~OIRAM_FLAG_ON_PIPE;
	}

	spt->vars.oiram.mode	= mode;
	spt->vars.oiram.state	= state;

	switch (mode)
	{
	case OIRAM_MODE_GROUND:
		spt->yVel = 0;
		switch (state)
		{
		case OIRAM_GROUND_STATE_IDLE:
			SptSetAnim (spt, OIRAM_ANIM_IDLE, FALSE);
			break;

		case OIRAM_GROUND_STATE_WALK:
			SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
			break;

		case OIRAM_GROUND_STATE_RUN:
			SptSetAnim (spt, OIRAM_ANIM_RUN, FALSE);
			break;

		case OIRAM_GROUND_STATE_DETECT:
			if (spt->xVel != 0)
			{
				if (KEYDOWN(KEY_RUN))
					OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_RUN, map);
				else
					OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_WALK, map);
			}
			else
			{
				OiramSetMode (spt, OIRAM_MODE_GROUND, OIRAM_GROUND_STATE_IDLE, map);
			}
			break;

		default:
			break;
		}
		break;

	case OIRAM_MODE_JUMP:
		spt->yVel = OIRAM_JUMP_VEL;
		spt->vars.oiram.timer = OIRAM_JUMP_TIME;
		SptSetAnim (spt, OIRAM_ANIM_JUMP, FALSE);
		break;

	case OIRAM_MODE_FALL:
		SptSetAnim (spt, OIRAM_ANIM_FALL, FALSE);
		break;

	case OIRAM_MODE_DIE:
		switch (state)
		{
		case OIRAM_DIE_STATE_DIE:
			spt->xVel = 0;
			spt->yVel = 0;
			spt->flags &= ~SPRITE_FLAG_COLLIDE;
			spt->vars.oiram.timer = OIRAM_DIE_TIME;
			SptSetAnim (spt, OIRAM_ANIM_DIE, FALSE);
			if (map != NULL)
			{
				map->followSprite = NULL;
			}
			break;

		case OIRAM_DIE_STATE_FALL:
			spt->xVel = 0;
			spt->yVel = OIRAM_DIE_FALL_SPEED;
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	case OIRAM_MODE_PIPE:
		SptSetAnim (spt, OIRAM_ANIM_PIPE, FALSE);
		spt->flags &= ~SPRITE_FLAG_COLLIDE;
		switch (state)
		{
		case OIRAM_PIPE_STATE_ENTER:
			spt->priority++;
			ASSERT (spt->priority < SPRITE_PRIORITY_NUM);
			spt->newX = spt->x = ((spt->x - spt->hotspotX) & ~(SPT_FX(TILE_SIZE)-1)) + spt->hotspotX;
			if (SptGroundSnap (spt, map, FALSE) == FALSE)
				spt->newX = spt->x = spt->x + SPT_FX(TILE_SIZE);
			spt->yVel = OIRAM_PIPE_SPEED;
			spt->vars.oiram.timer = OIRAM_PIPE_TIME;
			break;

		case OIRAM_PIPE_STATE_EXIT:
			spt->priority++;
			ASSERT (spt->priority < SPRITE_PRIORITY_NUM);
			spt->yVel = -OIRAM_PIPE_SPEED;
			spt->vars.oiram.timer = OIRAM_PIPE_TIME;
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		ASSERT_ALWAYS();
	}
}

static BOOL OiramGroundSnap (Sprite *spt, Map *map)
{
	if (map != NULL)
	{
		if (SptGroundSnap (spt, map, TRUE) == FALSE)
		{
			OiramSetMode (spt, OIRAM_MODE_FALL, OIRAM_FALL_STATE_FALL, map);
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}

	return FALSE;
}
