#include "Main.h"
#include "Input.h"
#include "Font.h"
#include "Map.h"
#include "MapData.h"
#include "Oiram.h"



// ----- Global functions -----

void SptInitOiram (Sprite *spt)
{
	spt->vars.oiram.mode = OIRAM_MODE_FALL;
	spt->vars.oiram.state = OIRAM_FALL_STATE_FALL;
}

void SptUpdateOiram (Sprite *spt)
{
	switch (spt->vars.oiram.mode)
	{
	case OIRAM_MODE_GROUND:
		if (KEYDOWN(KEY_LEFT))
		{
			spt->flags |= SPRITE_FLAG_HFLIP;
			spt->xVel = -SPT_FX(4);
			SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
		}
		else if (KEYDOWN(KEY_RIGHT))
		{
			spt->flags &= ~SPRITE_FLAG_HFLIP;
			spt->xVel = SPT_FX(4);
			SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
		}
		else
		{
			spt->xVel = 0;
			SptSetAnim (spt, OIRAM_ANIM_IDLE, FALSE);
		}

		if (KEYDOWN(KEY_UP))
			spt->yVel = -SPT_FX(4);
		else if (KEYDOWN(KEY_DOWN))
			spt->yVel = SPT_FX(4);
		else
			spt->yVel = SPT_FX(0);

		break;

	case OIRAM_MODE_JUMP:
		break;

	case OIRAM_MODE_FALL:
		spt->yVel += SPT_GRAVITY;
		if (spt->yVel > SPT_GRAVITY_MAX)
			spt->yVel = SPT_GRAVITY_MAX;

		if (KEYDOWN(KEY_LEFT))
		{
			spt->flags |= SPRITE_FLAG_HFLIP;
			spt->xVel = -SPT_FX(4);
			SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
		}
		else if (KEYDOWN(KEY_RIGHT))
		{
			spt->flags &= ~SPRITE_FLAG_HFLIP;
			spt->xVel = SPT_FX(4);
			SptSetAnim (spt, OIRAM_ANIM_WALK, FALSE);
		}
		else
		{
			spt->xVel = 0;
			SptSetAnim (spt, OIRAM_ANIM_IDLE, FALSE);
		}
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void SptColOiram (Sprite *spt, Sprite *obs, u32 dir)
{
	char text[4][16] =
	{
		{ "L" },
		{ "U" },
		{ "D" },
		{ "R" },
	};

	FontPrint8 (&gScreen, 4, 16, text[dir], FONT_COLORS(4, 254, 0));

	if (obs == NULL)	// Map collision
	{
		if (dir == SPRITE_COL_DIR_LEFT || dir == SPRITE_COL_DIR_RIGHT)
			spt->newX = spt->x;
		else if (dir == SPRITE_COL_DIR_UP || dir == SPRITE_COL_DIR_DOWN)
			spt->newY = spt->y;
	}
	else	// Sprite collision
	{
/*	if (dir == SPRITE_COL_DIR_DOWN)
	{
		// Stomp!
	}
	else
	{
		// Ouch!
		spt->flags &= ~SPRITE_FLAG_COLLIDE;
	}*/
	}
}
