// ----- Change log -----
// 02/27/05 - Created
// ----------------------

#ifndef OIRAM_H
#define OIRAM_H

#include "Main.h"



// ----- Constants -----

#define OIRAM_WALK_SPEED		SPT_FX(4)
#define OIRAM_RUN_ACCEL			SPT_FX(2)
#define OIRAM_RUN_SPEED			SPT_FX(8)
#define OIRAM_JUMP_VEL			SPT_FX(-12)
#define OIRAM_JUMP_TIME			10
#define OIRAM_DIE_TIME			15
#define OIRAM_DIE_FALL_SPEED	SPT_FX(-16)
#define OIRAM_DIE_FALL_ACCEL	SPT_FX(4)
#define OIRAM_HURT_TIME			30
#define OIRAM_PIPE_SPEED		SPT_FX(2)
#define OIRAM_PIPE_TIME			32
#define OIRAM_PAL_OVERALLS		227

extern RGB dOiramOverallsNormal;
extern RGB dOiramOverallsFlower;

typedef enum _OIRAM_FLAGS
{
	OIRAM_FLAG_FLOWER		= BIT00,
	OIRAM_FLAG_HURT			= BIT01,
	OIRAM_FLAG_ON_PIPE		= BIT02,

} OIRAM_FLAGS;

typedef enum _OIRAM_ANIM
{
	OIRAM_ANIM_IDLE,
	OIRAM_ANIM_WALK,
	OIRAM_ANIM_RUN,
	OIRAM_ANIM_JUMP,
	OIRAM_ANIM_FALL,
	OIRAM_ANIM_DIE,
	OIRAM_ANIM_PIPE,

	OIRAM_ANIM_NUM,
	OIRAM_ANIM_MAX = OIRAM_ANIM_NUM - 1,

} OIRAM_ANIM;

typedef enum _OIRAM_MODE
{
	OIRAM_MODE_GROUND,
	OIRAM_MODE_JUMP,
	OIRAM_MODE_FALL,
	OIRAM_MODE_DIE,
	OIRAM_MODE_PIPE,

	OIRAM_MODE_NUM,
	OIRAM_MODE_MAX = OIRAM_MODE_NUM - 1,

} OIRAM_MODE;

typedef enum _OIRAM_GROUND_STATE
{
	OIRAM_GROUND_STATE_IDLE,
	OIRAM_GROUND_STATE_WALK,
	OIRAM_GROUND_STATE_RUN,

	OIRAM_GROUND_STATE_NUM,
	OIRAM_GROUND_STATE_MAX = OIRAM_GROUND_STATE_NUM - 1,

		// Detect state to go to by velocity/run button held
	OIRAM_GROUND_STATE_DETECT = OIRAM_GROUND_STATE_NUM,

} OIRAM_GROUND_STATE;

typedef enum _OIRAM_JUMP_STATE
{
	OIRAM_JUMP_STATE_JUMP,

	OIRAM_JUMP_STATE_NUM,
	OIRAM_JUMP_STATE_MAX = OIRAM_JUMP_STATE_NUM - 1,

} OIRAM_JUMP_STATE;

typedef enum _OIRAM_FALL_STATE
{
	OIRAM_FALL_STATE_FALL,

	OIRAM_FALL_STATE_NUM,
	OIRAM_FALL_STATE_MAX = OIRAM_FALL_STATE_NUM - 1,

} OIRAM_FALL_STATE;

typedef enum _OIRAM_DIE_STATE
{
	OIRAM_DIE_STATE_DIE,
	OIRAM_DIE_STATE_FALL,

	OIRAM_DIE_STATE_NUM,
	OIRAM_DIE_STATE_MAX = OIRAM_DIE_STATE_NUM - 1,

} OIRAM_DIE_STATE;

typedef enum _OIRAM_PIPE_STATE
{
	OIRAM_PIPE_STATE_ENTER,
	OIRAM_PIPE_STATE_EXIT,

	OIRAM_PIPE_STATE_NUM,
	OIRAM_PIPE_STATE_MAX = OIRAM_PIPE_STATE_NUM - 1,

} OIRAM_PIPE_STATE;



// ----- Structures -----

typedef struct _SptVarsOiram
{
	u8 mode;
	u8 state;
	u8 timer;
	u8 hurtTimer;
	u8 flags;		// OIRAM_FLAGS

} SptVarsOiram;



// ----- Global functions -----

extern void SptInitOiram (struct _Sprite *spt, struct _Map *map);
extern void SptUpdateOiram (struct _Sprite *spt, struct _Map *map);
extern void SptRenderOiram (struct _Sprite *spt, struct _Map *map, struct _Image *screen);
extern void SptColOiram (struct _Sprite *spt, struct _Sprite *obs, u32 dir, struct _Map *map);

#endif	// OIRAM_H