#include "Main.h"
#include "Map.h"
#include "MapData.h"
#include "Turtle.h"



// ----- Local function prototypes -----

static void TurtleSetMode (Sprite *spt, u32 mode, u32 state);
static void TurtleTurnAround (Sprite *spt);



// ----- Global functions -----

void SptInitTurtle (Sprite *spt, Map *map)
{
	TurtleSetMode (spt, TURTLE_MODE_GROUND, TURTLE_GROUND_STATE_WALK);
}

void SptUpdateTurtle (Sprite *spt, Map *map)
{
	switch (spt->vars.oiram.mode)
	{
	case TURTLE_MODE_GROUND:
		{
			s32 ofs;

			if (spt->xVel > 0)
				ofs = SPT_FX(TILE_SIZE);
			else
				ofs = SPT_FX(-TILE_SIZE);

			spt->x += ofs;

			if (SptGroundSnap (spt, map, FALSE) == FALSE)
			{
				TurtleTurnAround (spt);
			}

			spt->x -= ofs;
		}
		break;

	case TURTLE_MODE_DIE:
		if (--spt->vars.turtle.timer == 0)
			spt->flags &= ~SPRITE_FLAG_ALIVE;
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void SptColTurtle (Sprite *spt, Sprite *obs, u32 dir, Map *map)
{
	if (obs == NULL)	// Map collision
	{
		TurtleTurnAround (spt);
	}
	else	// Sprite collision
	{
		if (SPRITE_IS_OIRAM(obs))
		{
			if (dir == SPRITE_COL_DIR_UP)
				TurtleSetMode (spt, TURTLE_MODE_DIE, TURTLE_DIE_STATE_DIE);
		}
		else if (SPRITE_IS_BADDIE(obs))
		{
			TurtleTurnAround (spt);
		}
		else if (SPRITE_IS_ATTACK(obs))
		{
			TurtleSetMode (spt, TURTLE_MODE_DIE, TURTLE_DIE_STATE_DIE);
		}
	}
}



// ----- Local functions -----

static void TurtleSetMode (Sprite *spt, u32 mode, u32 state)
{
	ASSERT (mode < TURTLE_MODE_NUM);

	spt->vars.turtle.mode	= mode;
	spt->vars.turtle.state	= state;

	switch (mode)
	{
	case TURTLE_MODE_GROUND:
		spt->xVel = TURTLE_WALK_SPEED;
		SptSetAnim (spt, TURTLE_ANIM_WALK, FALSE);
		break;

	case TURTLE_MODE_DIE:
		spt->xVel = 0;
		spt->yVel = 0;
		spt->flags &= ~SPRITE_FLAG_COLLIDE;
		SptSetAnim (spt, TURTLE_ANIM_DIE, FALSE);
		spt->vars.turtle.timer = TURTLE_DIE_TIME;
		break;

	default:
		break;
	}
}

static void TurtleTurnAround (Sprite *spt)
{
	spt->flags ^= SPRITE_FLAG_HFLIP;
	spt->xVel = -spt->xVel;
}
